# Copyright (C) 2020 Ludvig Larsson aka Valmond ludviglarsson@protonmail.com
# This code is licensed under the terms of the GNU Public License, version 3 or later (GPL-3.0-or-later)

from sys import exit
import threading
import signal
import socket
import datetime

"""
This service provides downloading help for the web browser extension.
As a web extension cannot, without taking crazy liberties with security, directly access executables or python scripts
we here provide a listener that will take download commands, and try to download the tenfingers payload.
It will run on IPV4 on localhost on port 47000

It basically does this:
./10f linkfile.10f
and reports back if it worked or not

Protocol:
download "folder"
"""

# Global shut down boolean
force_shutdown = False


class Listen:
    def __init__(self):
        self.listen_socket = None
        self.client_socket = None

    def force_stop(self):
        # logging.info("Shutting down listener...")
        if self.listen_socket is not None:
            self.listen_socket.close()

    @staticmethod
    def listen_thread(client_socket, client_address):
        print("Recv Message from" + str(client_address[0]))

        raw_input = client_socket.recv(1024)
        if raw_input:
            query = raw_input.decode()
            print(query)

        client_socket.send(b"Yolo!")

        # Close the socket
        client_socket.close()

    def run(self):
        port = 47000
        print('inner thread start')
        try:
            # Here we could/should(?) list all network interfaces and their supported protocols...

            self.listen_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            self.listen_socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            self.listen_socket.bind(('localhost', port))

            self.listen_socket.listen(20)  # max simultaneous connexions
            self.listen_socket.settimeout(30)

        except socket.error as err:
            print('Failed to create listen socket: ', err)
            exit()

        # log
        print('Listen on port: ' + str(port))

        global force_shutdown
        while not force_shutdown:
            try:
                # establish connection
                client_socket, client_address = self.listen_socket.accept()

                # Make a new thread that will deal with the communication:
                if not force_shutdown:
                    # Read request:
                    client_socket.settimeout(60)
                    # Start listener
                    threading.Thread(target=self.listen_thread, args=(client_socket, client_address)).start()

            except socket.error:  # as err:
                # print("listen exception", err)
                pass


version = '1.0'

print('Tenfingers web browser downloader is starting up: V' + version + ' start time: ' + str(datetime.datetime.now()))
# todo: add port parameter to args


# Create thread and start up:
listener = Listen()
listener_thread = threading.Thread(name='Listener', target=listener.run)  # l.run2, args=("abc",))

print("start listener thread:")
listener_thread.start()


# Set up a function that will be called at SIGINT so that a CTRL+C will shut down all threads and exit correctly:
def signal_handler(sig, frame):
    print('Received SIGINT, shutting down threads.')
    global force_shutdown
    force_shutdown = True
    listener.force_stop()


# Set up the SIGINT handler
signal.signal(signal.SIGINT, signal_handler)
